import { isElement } from "./utils";
import { setAttrs } from "./attr";
import { setStyles } from "./css";
function parseHtml(html) {
    const parser = new DOMParser();
    const doc = parser.parseFromString(html, "text/html");
    const children = doc.body.children;
    if (children.length !== 1) {
        throw new Error("Can only parse HTML with exactly one valid root element. A valid element can stand on its own in the body.");
    }
    return children[0];
}
export function createElement(type, attributes = {}) {
    const element = createUnstyledElement(type, attributes);
    if (attributes.css !== undefined) {
        setStyles(element, attributes.css);
    }
    return element;
}
export function createUnstyledElement(type, attributes = {}) {
    const element = (-1 !== type.indexOf("<"))
        ? parseHtml(type)
        : document.createElement(type);
    setAttrs(element, attributes);
    if (attributes.text !== undefined) {
        element.textContent = attributes.text;
    }
    else if (attributes.html !== undefined) {
        element.innerHTML = attributes.html;
    }
    return element;
}
export function remove(element) {
    if (null === element) {
        return;
    }
    const list = Array.isArray(element) ? element : [element];
    for (let i = 0; i < list.length; i++) {
        let parentNode = list[i].parentNode;
        if (null !== parentNode) {
            parentNode.removeChild(list[i]);
        }
    }
}
export function empty(element) {
    const list = Array.isArray(element) ? element : [element];
    for (let i = 0; i < list.length; i++) {
        if (isElement(list[i])) {
            list[i].innerHTML = "";
        }
    }
}
export function replace(element, replacement) {
    let parentNode = element.parentNode;
    if (null !== parentNode) {
        parentNode.replaceChild(replacement, element);
    }
}
function insertElement(reference, insert, adjacentPosition, insertInto, insertReference) {
    if (typeof insert === "string") {
        reference.insertAdjacentHTML(adjacentPosition, insert);
        return;
    }
    if (insertInto === null) {
        return;
    }
    const list = Array.isArray(insert) ? insert : [insert];
    for (let i = 0; i < list.length; i++) {
        insertInto.insertBefore(list[i], insertReference);
    }
}
export function append(reference, insert) {
    insertElement(reference, insert, "beforeend", reference, null);
}
export function prepend(reference, insert) {
    insertElement(reference, insert, "afterbegin", reference, reference.firstElementChild);
}
export function before(reference, insert) {
    insertElement(reference, insert, "beforebegin", reference.parentElement, reference);
}
export function after(reference, insert) {
    insertElement(reference, insert, "afterend", reference.parentElement, reference.nextElementSibling);
}
export function toggleClass(element, classNames, add) {
    let classes = Array.isArray(classNames) ? classNames : [classNames];
    classes.forEach(className => element.classList[add ? "add" : "remove"](className));
}
