export interface RequestOptions {
    method?: string;
    headers?: {
        [name: string]: string;
    };
    data?: BodyInit | null;
    json?: null | {
        [name: string]: any;
    } | Array<any>;
    credentials?: boolean;
}
export interface SuccessResponse<T> {
    response: Response;
    success: boolean;
    data: T;
}
export declare type FailureResponse = RequestFailureResponse | JsonFailureResponse | StatusFailureResponse;
export interface RequestFailureResponse {
    error: Error;
    reason: "request_failed";
}
export interface JsonFailureResponse {
    error: Error;
    response: Response;
    reason: "invalid_json";
}
export interface StatusFailureResponse {
    error: Error;
    response: Response;
    data: any;
    reason: "status";
}
export declare function isFailureResponse(response: any): response is FailureResponse;
export declare function isRequestFailure(response: any): response is RequestFailureResponse;
export declare function isJsonFailure(response: any): response is JsonFailureResponse;
export declare function isStatusFailure(response: any): response is StatusFailureResponse;
export declare function request<T extends object = {}>(url: string, options?: RequestOptions): Promise<SuccessResponse<T>>;
