import { extend } from "../extend";
export class FetchClient {
    constructor(toasts, loader = null, failureMessage, defaultLoadingMessage) {
        this.toasts = toasts;
        this.loader = loader;
        this.failureMessage = failureMessage || "Die Anfrage ist leider fehlgeschlagen. Bitte lade die Seite neu und versuche es noch einmal.";
        this.defaultLoadingMessage = undefined !== defaultLoadingMessage
            ? defaultLoadingMessage
            : ["Lädt... bitte warten"];
    }
    request(url, options = {}) {
        const headers = extend(options.headers || {}, {
            Accept: "application/json",
            "X-Requested-With": "XMLHttpRequest",
        });
        let data = options.data;
        let method = options.method;
        const showGenericRequestErrors = false !== options.handleGenericRequestErrors;
        const showLoader = null !== options.loading;
        if (showLoader && this.loader) {
            const loadingMessages = options.loading || this.defaultLoadingMessage;
            this.loader.start(...loadingMessages);
        }
        if (options.json) {
            data = JSON.stringify(options.json);
            headers["Content-Type"] = "application/json; charset=UTF-8";
        }
        if (!method) {
            method = data ? "post" : "get";
        }
        return new Promise((resolve, reject) => {
            fetch(url, {
                body: data,
                cache: "no-cache",
                credentials: "include",
                headers: headers,
                method: method,
            })
                .then(raw => {
                return raw.json().then((response) => {
                    const message = response.message;
                    if (message) {
                        this.toasts.add(message.text, message.impact, message.action);
                    }
                    if (response.redirect) {
                        document.location.href = response.redirect;
                    }
                    resolve({
                        ok: response.ok,
                        status: response.status,
                        data: response.data,
                    });
                }, error => {
                    if (showGenericRequestErrors) {
                        console.error("Request failed (json)", error);
                        this.toasts.add(this.failureMessage, "negative");
                    }
                    reject();
                });
            }, error => {
                if (showGenericRequestErrors) {
                    console.error("Request failed (generic)", error);
                    this.toasts.add(this.failureMessage, "negative");
                }
                reject();
            })
                .finally(() => {
                if (showLoader && this.loader) {
                    this.loader.end();
                }
            });
        });
    }
}
