import { off, on } from "./dom/events";
import { isChildElement } from "./dom/traverse";
export function registerBodyClickHandler(allowedClickTargets, onInvalidTargetClick) {
    let handler = (event) => {
        for (let i = 0; i < allowedClickTargets.length; i++) {
            if (isChildElement(allowedClickTargets[i], event.target)) {
                return;
            }
        }
        onInvalidTargetClick();
    };
    on(document.body, "click", handler);
    return () => {
        off(document.body, "click", handler);
    };
}
export function initDismissableContainer(trigger, allowedContainers, callback) {
    let globalHandler = null;
    const triggers = Array.isArray(trigger) ? trigger : [trigger];
    const close = () => {
        if (globalHandler) {
            globalHandler();
            globalHandler = null;
        }
        callback(false);
    };
    const open = () => {
        if (globalHandler) {
            return;
        }
        globalHandler = registerBodyClickHandler(triggers.concat(allowedContainers), close);
        callback(true);
    };
    const eventListener = () => {
        (globalHandler ? close : open)();
    };
    on(triggers, "click", eventListener);
    close.destroy = () => {
        close();
        off(triggers, "click", eventListener);
    };
    return close;
}
