function typeError(expected, value) {
    throw new Error(undefined === value
        ? `Missing parameter, expected ${expected}`
        : `Can't parse value as ${expected}: '${value}'`);
}
export function stringParameter(defaultValue) {
    return (value) => {
        if (null == value) {
            return defaultValue === undefined
                ? typeError("string", value)
                : defaultValue;
        }
        return "" + value;
    };
}
export function numberParameter(defaultValue) {
    return (value) => {
        if (typeof value === "number") {
            return value;
        }
        const parsed = parseInt(value, 10);
        if (!isNaN(parsed)) {
            return parsed;
        }
        return (defaultValue === undefined || null != value)
            ? typeError("number", value)
            : defaultValue;
    };
}
export function booleanParameter(defaultValue) {
    return (value) => {
        if (typeof value == "boolean") {
            return value;
        }
        const isTrue = "true" === value;
        if ("false" === value || isTrue) {
            return isTrue;
        }
        return (defaultValue === undefined || null != value)
            ? typeError("boolean", value)
            : defaultValue;
    };
}
